;-------T-------T------------------------T---------------------------------;
;Open a screen using an external OBJect file.
;--------------------------------------------
;This demo will open a screen, based on the parameters in an external object
;file.  This allows a VERY high level of external editing that will alter
;how this program behaves, without having to reassemble it.
;
;This feature is highly important as it allows up to 100% configurability of
;your game, which previously has only been available to applications using
;libraries such as MUI.  However, it is even more powerful than this as
;graphic artists could not only create new datasets for your game, but also
;alter the dimensions, colours, resolution, attributes (and more) of your
;BOBs and screens.  Support for PC relative CODE segments is also available,
;so any external functions could be altered and improved by programmers.
;This will ensure that your game has a long lasting future as it keeps up
;with current technology and additions to GMS.
;
;I hope you make the most of external objects, its worth it!

	INCDIR	"INCLUDES:"
	INCLUDE	"exec/exec_lib.i"
	INCLUDE	"games/games_lib.i"
	INCLUDE	"games/games.i"

CALL	MACRO
	jsr	_LVO\1(a6)
	ENDM

	SECTION	"ObjectTags",CODE

;===========================================================================;
;                             INITIALISE DEMO
;===========================================================================;

Start:	MOVEM.L	A0-A6/D1-D7,-(SP)
	move.l	($4).w,a6
	lea	GMS_Name(pc),a1
	moveq	#$00,d0
	CALL	OpenLibrary
	move.l	d0,GMS_Base
	beq	.Error_GMS

	move.l	GMS_Base(pc),a6
	sub.l	a0,a0
	CALL	SetUserPrefs

	lea	OBJName(pc),a0
	CALL	LoadObjectFile
	move.l	d0,ObjectBase
	beq.s	.Error_ObjectFile

	move.l	ObjectBase(pc),a0	;a0 = Object Base.
	lea	TXT_ScreenName(pc),a1	;a1 = The name of the screen to get.
	moveq	#$00,d0	;d0 = ID (NULL ignores ID settings).
	CALL	GetObject	;>> = Get our GameScreen object.
	tst.l	d0	;d0 = Check if found.
	beq.s	.Error_Screen	;>> = Quit if error.

	move.l	d0,a0	;a0 = Screen Tags of our object.
	CALL	AddScreen	;>> = Initialise the screen.
	move.l	d0,Screen	;MA = Save pointer to the GameScreen.
	beq.s	.Error_Screen	;>> = Quit if error.

	move.l	Screen(pc),a0
	lea	Picture(pc),a1
	move.l	GS_MemPtr1(a0),PIC_Data(a1)
	CALL	LoadPic
	tst.w	d0
	bne.s	.Error_Picture

	move.l	PIC_Palette(a1),GS_Palette(a0)
	CALL	UpdatePalette

	CALL	ShowScreen

	bsr.s	Main

.ReturnToDOS
	move.l	GMS_Base(pc),a6
	lea	Picture(pc),a1
	CALL	FreePic
.Error_Picture
	move.l	Screen(pc),a0
	CALL	DeleteScreen
.Error_Screen
	move.l	ObjectBase(pc),a0
	CALL	FreeObjectFile
.Error_ObjectFile
	move.l	GMS_Base(pc),a1
	move.l	($4).w,a6
	CALL	CloseLibrary
.Error_GMS
	MOVEM.L	(SP)+,A0-A6/D1-D7
	moveq	#$00,d0
	rts

;===========================================================================;
;                                MAIN LOOP
;===========================================================================;

Main:	CALL	WaitSVBL
	moveq	#JPORT1,d0
	CALL	ReadMouse
	btst	#MB_LMB,d0
	beq.s	Main
	rts

;===========================================================================;
;                                  DATA
;===========================================================================;

GMS_Name:
	dc.b	"games.library",0
	even
GMS_Base:
	dc.l	0

ObjectBase:
	dc.l  0
OBJName	dc.b  "GAMESLIB:data/OBJ.ScreenTags"
Screen:	dc.l  0
TXT_ScreenName:
	dc.b	"DemoScreen",0
	even

Picture	dc.l	PCV1,0	;Version header.
	dc.l	0	;Source data.
	dc.w	640,0,256	;Width, Height.
	dc.w	4	;Amount of Planes.
	dc.l	16	;Amount of colours.
	dc.l	0	;Source palette (remap).
	dc.w	HIRES|COL12BIT	;Screen mode.
	dc.w	ILBM	;Destination
	dc.l	GETPALETTE	;Parameters.
	dc.l	.File
.File	dc.b	"GAMESLIB:data/IFF.Pic640x256",0
	even
